(* $Id: gont.mli,v 1.10 2001/11/12 09:09:25 malekith Exp $ *)

(* Identifier. *)
type id = string

type location = {mutable file: string; mutable line: int}

(* Types used by gont compiler. *)
type ty =
    Int                         (* integer of the same size as pointer *)
  | Void                        (* unit type *)
  | Bool                        (* boolean, same size as Int *)
  | Float                       (* C's double *)
  (* Next two types are very similar from the language sementics point
   * of view, however they are something very distinct for the compiler.
   * RealFunc(return_type, argument_types list) is used for functions that 
   * are actually generated. We call them indirectly. They are not mutable.
   * This is very similar to function from C's point of view. Func(...)
   * on the other hand is void* pointer to datastructure describing how
   * call particular function. Variables of this type can be modified,
   * passed arround and partially appalied. Partial application is reason
   * why they has to be pointer to datastructure, and not simply pointer
   * to function.
   *
   * When it is used in type context (for example return value from type_exp
   * rule in grammar), Func(...) is used. Similary in typechecker.
   * RealFunc(...) should be only seen in symbol table.
   *)
  | Func of ty * (ty list)
  | RealFunc of ty * (ty list)
  | String

type raw_exp =
    SimpleBinOp of Ksi.binop * exp * exp
  | SimpleUnOp of Ksi.unop * exp
  | Var of id
  | Call of exp * exp list
  | IntConst of string
  | BoolConst of bool
  | FloatConst of string
  | StringConst of string
and exp = {mutable raw_exp: raw_exp; 
           mutable exp_ty: ty option; 
           exp_loc: location}
and raw_stmt =
    Skip
  | Exp of exp
  | Return of exp
  | ReturnVoid
  | Block of stmt list
and stmt = {raw_stmt: raw_stmt;
            stmt_loc: location}

type storage =
    Local
  | Public

type fn_decl = {fn_storage: storage;
                fn_ret_type: ty; 
                fn_name: id; 
                fn_parms: (ty * id) list;
                fn_body: stmt;
                fn_loc: location}

type extern_c_decl = {xc_ret_type: ty; 
                      xc_name: id; 
                      xc_parms: ty list;
                      xc_loc: location}
type decl =
    FuncDecl of fn_decl
  | GlobalDecl of storage * ty * id * location
  | ExternC of extern_c_decl


exception Syntax_error of location * string
exception Eof
;;

(* Check whatever t is subtype of s (t < s). At this very moment this 
 * checks for type equality... *)
val subtype : (* t *) ty -> (* s *) ty -> bool

(* Return textual representation of given type. *)
val string_of_type : ty -> string

(* Return corresponding Ksi type (to be inserted to output stream). *)
val ksi_type : ty -> Ksi.ty

(* Generate Ksi code for given Gont program. *)
val codegen : decl -> Ksi.decl list
;;

val debug : string -> unit

(* An alias for Printf.sprintf. *)
val xf : ('a, unit, string) format -> 'a

(* Display warning. *)
val warn : location * string -> unit

(* Display error message. *)
val err_msg : location * string -> unit

(* Raise Syntax_error. *)
val err : location * string -> 'a
;;


class symbol : ty * id ->
  object
    method get_name : id
    method get_type : ty
  end

class compenv :
  object
    (* Find symbol in current scope and up. *)
    method lookup_symbol : id -> symbol

    (* Check whatever given symbol exists in current scope and up or
     * just in current scope 
     *)
    method symbol_exists : id -> bool
    method symbol_exists_in_current_scope : id -> bool
    
    (* Add symbol to current scope. *)
    method add_symbol : symbol -> unit
    
    (* Enter nested scope. *)
    method push_level : unit
    
    (* Leave nested scope. *)
    method pop_level : unit
    
    (* Add toplevel Ksi declaration to be dumped before dumping
     * Ksi declaration currently under construction.
     * Used to output local helper functions etc. 
     *)
    method add_decl : Ksi.decl -> unit

    (* Get Ksi declarations stored with add_decl. Used internally
     * by codegen.
     *)
    method get_decls : Ksi.decl list
    
    (* Return unique name prefixed with string. String should start
     * with '@', unless it is going to get exported. 
     *)
    method seq_name : id -> id

    method set_fn_name : id -> unit
    method fn_name : id
    method set_fn_ret_type : ty -> unit
    method fn_ret_type : ty
    method fn_finish : unit
  end
  
val env : compenv
;;
