(* $Id: lexer.mll,v 1.9 2001/11/12 09:09:25 malekith Exp $ *)

(* Lexer for the gont language. *)

{

open Gont
open Parser        (* The type token is defined in parser.mli *)
;;

let comment_depth = ref 0
let comment_start = {file = ""; line = 0}
let current_location = {file = "stdin"; line = 1}
;;

let keywords = Hashtbl.create 50;;

List.iter (fun (s, t) -> Hashtbl.add keywords s t)
    [ 
      ("int", KW_INT);
      ("return", KW_RETURN);
      ("void", KW_VOID);
      ("skip", KW_SKIP);
      ("bool", KW_BOOL);
      ("float", KW_FLOAT);
      ("public", KW_PUBLIC);
      ("string", KW_STRING);
      ("extern_c", KW_EXTERN_C)
    ]
;;

let process_id id =
  try Hashtbl.find keywords id with Not_found -> ID id

let err s = raise(Syntax_error (current_location, s))

let next_line () = 
  current_location.line <- current_location.line + 1
;;

(* Process \'s in string. Discard first and last '"' *)
let process_string s =
  let r = String.create (String.length s) in
  let seen_close = ref false in
  let i = ref 0 in
  let st = Stream.of_string s in
    ((Stream.junk st);
     (Stream.iter 
          (fun c -> 
            if !seen_close then 
              assert false
            else if c == '"' then
              seen_close := true
            else
              (r.[!i] <- (match c with
                            '\\' -> 
                                (match Stream.next st with
                                    'n' -> '\n'
                                  | 'r' -> '\r'
                                  | 'b' -> '\b'
                                  | x -> x)
                           | x -> x);
               incr i))
          st);
     STRING(String.sub r 0 !i))

(* exported *)
let get_current_location () = current_location
;;

}

let nl = ('\r' '\n' | '\n' | '\r')

let num = ['0'-'9']

rule token = parse
    [' ' '\t']          { token lexbuf }     (* skip blanks *)
  | nl                  { next_line (); token lexbuf }
  | ((num+ '.' num+ 'e' (['+' '-']?) num+) | (num+ 'e' (['+' '-']?) num+))
                        { FLOAT(Lexing.lexeme lexbuf) }
  | (num+ | '0' ['x' 'X'] ['0'-'9''a'-'f''A'-'F']+ | '0' ['b' 'B'] ['0' '1']+)
                        { INT(Int64.to_string (Int64.of_string(Lexing.lexeme lexbuf))) }
  | ['a'-'z''A'-'Z''_']['a'-'z''A'-'Z''_''0'-'9']*
                        { process_id(Lexing.lexeme lexbuf) }
  | '"' ([ ^ '"' ] | '\\' [ '\\' '"' ])* '"'
                        { process_string(Lexing.lexeme lexbuf) }
  | "/*"                { comment_depth := 1;
                          comment_start.file <- current_location.file;
                          comment_start.line <- current_location.line;
                          comment lexbuf; 
                          token lexbuf }
  | "//" [^ '\n' '\r']* nl { next_line (); token lexbuf }
  | "->"                { ARROW }
  | '='                 { EQUAL }
  | '+'                 { PLUS }
  | '-'                 { MINUS }
  | '*'                 { TIMES }
  | '/'                 { DIV }
  | '('                 { LPAREN }
  | ')'                 { RPAREN }
  | '{'                 { LBRACE }
  | '}'                 { RBRACE }
  | ';'                 { SEMICOLON }
  | ','                 { COMMA }
  | '['                 { LSQUARE }
  | ']'                 { RSQUARE }
  | eof                 { raise Eof }
  | _                   
    { err ("invalid character in input `" ^ (String.escaped (Lexing.lexeme lexbuf)) ^ "'") }

and comment = parse
    "*/"		{ decr comment_depth; 
                          if !comment_depth > 0 then (comment lexbuf) }
  | "/*"                { incr comment_depth; comment lexbuf }
  | nl                  { next_line (); comment lexbuf }
  | eof                 
   { raise (Syntax_error (comment_start, "EOF within comment")) }
  | _			{ comment lexbuf }

